/** \mainpage Présentation générale du module GEST

\author Brice Burger


\section a Objectifs du module

Ce module vise à suivre la tête et les deux mains, en 3D, d'un homme en face d'un robot. Pour se faire, il prend en entrée les images d'un systèmes de caméras stéréo (actuellement fournies par le module viam) afin d'en extraire différentes détections et mesures. Ces dernières permettent, grâce à du filtrage particulaire, d'initialiser automatiquement les cibles du traqueur (tête et mains), puis de les suivre.

\section b Compilation

\subsection truc Prérequis :
\li opencv doit être présent dans le système. Il est facile à installer via yum:
yum install opencv-devel
\li les modules POM et VIAM (ou CAMERA) sont requis par ce module.

\subsection c Compilation du code de base :
cd code<br>
make clean<br>
make depend<br>
make<br>
cd -

\subsection d Compilation du module genom :
genom -t gest<br>
mkdir build<br>
cd build<br>
../configure --prefix=... # à compléter correctement bien entendu...<br>
make<br>
make install


\section e Organisation du code

Le code du module Genom est contenu dans le répertoire "codels".
Tout le reste du code est contenu dans le répertoire "code", qui en réalité une librairie utilisable en dehors du module.

\subsection f Dans le répertoire "code"
\li src contient les sources
\li make.defs permet de configurer la compilation du module (à modifier si vous souhaitez modifier le nom ou l'emplacement de la librairie, utiliser le mode DEBUG, etc)
\li fichiers_donnees contient un ensemble de fichiers de données, dont les fichiers de modélisation de la couleur peau
\li lib, une fois le code compilé, contiendra les librairies
\li include, une fois le code compilé, contiendra les headers du code
\li doc permet la génération de la présente documentation

\subsection g Dans le sous-répertoire "src"
\li detection : ce répertoire contient le code des détecteurs utilisés pour le traqueur, mais aussi, comme son nom ne l'indique pas, des mesures et des fonctions de triangulation et projection. Les mesures et détections sont organisées par type (un dossier pour chacun). Pour chaque type de mesure ou détection, on trouvera deux fichiers/classes, l'une définissant son utilisation image (2D), l'autre permettant son extension à la 3D. Dans ce dossier, le fichier detectionParams.h contient les constantes globales liées à cette partie du code.
\li gest : ce répertoire contient le code directement lié au filtrage particulaire : le code général du traqueur, les méthodes d'évaluation générale et tout ce qui a trait aux particules du filtre (génération, etc). Dans ce dossier, le fichier gestParams.h contient les constantes globales liées à cette partie du code.
\li LeMain : ce répertoire contient le main.cpp qui permet notamment d'utiliser le module hors-ligne.
\li utils : ce répertoire contient diverses fonctions utilitaires dont la plupart ont été héritées de Ludovic Brethes. En réalité, peu servent et serait temps d'y faire un grand ménage...

\section r Utilisation du module

\subsection ra Utilisation autonome (hors-ligne, hors module genom)

Le code peut fonctionner hors-ligne, en dehors du module genom. Pour obtenir un exécutable :<br>
cd code<br>
make main (compilation normale) ou :<br>
make remain (force une recompilation totale)<br>

Des exemples d'utilisation de l'exécutable sont donnés dans le fichier src/LeMain/main.cpp . Le principe est de lancer le suivi sur une séquence préenregistrée (par le module GEST par exemple).<br>

Cet exécutable sert également à apprendre une couleur ou à calculer des taux de réussite du suivi, par exemple. Voir le fichier pour en savoir plus.

\subsection rb Utilisation du module genom

\subsubsection rc Lancer le module et autres commandes de base

\li démarrer les modules GEST et camera (ou viam) au minimum
\li initialiser le module camera (ou viam)
\li initialiser le module GEST :<br>
: :gest::Init
\li OU Initialiser le module GEST avec utilisation en parallèle du module GAZE :<br>
: :gest::InitForGaze
\li Réglage des paramètres (ils peuvent également être modifiés pendant que le suivi fonctionne)<br>
: :gest::SetTrackingParams : précise l'espace de couleur à utiliser et si on utilise une séquence d'image préenregistrées (à éviter, utiliser directement le code hors-ligne pour ce genre de choses) ou directement l'entrée de VIAM<br>
: :gest::SetGraphicParams : précise le type d'interface graphique souhaitée, si l'image supplémentaire doit être celle de gauche ou de droite, si l'on veut voir la projection de toutes les particules et si l'image en entrée est débayérisée ou non.<br>
\li Lancer le suivi :<br>
: :gest::Track
\li Arrêter le suivi :<br>
: :gest::Stop
\li tester uniquement l'initalisation du module :<br>
: :gest::LoopInitialisation
\li déplacement de l'affichage sur l'écran :
: :gest::WindowPlacement

\subsubsection rd Sauvegarder une séquence pour traitement hors-ligne par GEST (i.e. une séquences d'images + configuration du robot) :
\li démarrage : SaveSequence "/chemin/sauvegarde/" GEST_True GEST_True
\li arrêt : Stop

\subsubsection re Sauvegarder le résultat du tracking (i.e. images résultats + fichier résultat complet)
\li démarrage : SaveTracking "/chemin/sauvegarde/" GEST_True GEST_True
\li arrêt : Stop

\subsubsection rf Les posters
\li : :gest::PositionposPosterRead : poster donnant l'état du suivi et la position de la tête et des 2 mains suivies dans le repère de la caméra.
\li : :gest::PositionposMainPosterRead : même poster que le précédent, mais dont les positions sont données dans le repère MAIN.
\li : :gest::PositionHeadDetectedPosterRead : si une tête est détectée (ATTENTION, le suivi de la tête peut fonctionner, même si elle n'est pas détectée).
\li : :gest::PositionconfigPosterRead : copie du poster POM_SENSOR_POS donné par VIAM au moment de la prise des images traitées par GEST.
\li : :gest::Positionpos2DPosterRead : position 2D (projection du suivi) de la tête dans l'image et son rayon. Ce poster sert notamment au fonctionnement du module GAZE.


\subsection rg Pour déboguer :

Il y a un mode de compilation "débogage". Par défaut, ce mode est activé quand on compile le code seul ("make main" ou "make remain") et désactivé quand on compile le code pour le module genom ("make", "make genom" ou "make regen").<br>
Si le mode "débogage" est activé, le code est compilé avec les options gdb et gprof. Pour utiliser gprof :<br>
src/LeMain/traqueur options...<br>
gprof src/LeMain/traqueur > gprof.output<br>
Les informations sont stockées dans gprof.output .


\section h Conventions

\subsection i Conventions visuelles

\li Une image barrée d'un trait noir signifie que rien n'est suivi, le traqueur cherche donc à s'initialiser.
\li Si les cibles sont suivies, elles sont modélisées de la manière suivante : la tête par une sphère de rayon fixe (matérialisée par un cercle blanc sur l'image), les mains par des ellipsoïdes déformables (matérialisée par des ellipses violette et jaune, respectivement pour la main droite et gauche).
\li Une cible barrée de sa propre couleur signifie qu'elle est temporairement perdue (poids trop faible).
\li Une cible qui n'est plus suivie (totalement perdue), disparait. Le traqueur cherchera alors à la réinitialiser.

\subsection j Conventions dans le code

Les conventions prises dans le code peuvent différer de celles du gestRunTaskCodels.cc.

\li angles en degré
\li distances en millimètres
\li dans le code, le repère de la caméra est : x à gauche, y vers le haut, z la profondeur (ATTENTION!! : différent dans gestRunTaskCodels.cc, et dans les sorties standard des modules où l'on suit les conventions robotiques : x vers l'avant et z vers le haut)
\li paramètres des mains (ellipsoïde) (3 translations, taille des demi-axes, 3 angles)=(tx, ty, tz, ax, ay, az, alpha, betha, gamma)
\li paramètres de la tête (sphère) (3 translations)=(tx, ty, tz)
\li les translations sont la position du repère de la cible dans le repère caméra
\li les demi-axes sont donnés suivant les axes du repère de la cible
\li les angles sont les rotations à effectuer successivement et dans l'ordre par chaque axe du repère de la cible par rapport au repère de la caméra
\li les demi-axes sont obtenus par svd, par conséquent : ax est toujours suivant la profondeur, ay suivant le coté visible le plus long et les angles permettent de retrouver ce repère
\li la taille des demi-axes est régulée (tailleBlobMin < a* < tailleBlobMax, définis dans detectionParams.h)
\li le volume des mains est fixé (demivolumeMain définis dans detectionParams.h), par conséquent : ax = demivolumeMain / (ay * az)

\section k Règles d'utilisation

\subsection l Limitations

\li À l'initialisation, il ne devrait y avoir qu'une seule personne en face du robot. Si ce n'est pas le cas, le module initialisera sur la première personnes vue. Après ce premier contact, d'autres personnes peuvent entrer dans le champs de vue des caméras.
\li L'utilisateur doit porter des manches longues : le traqueur a beaucoup de mal à savoir où s'arrête la main et commence le bras!
\li Il faut éviter d'avoir derrière soit des objet de couleur chaire.

\subsection m Savoir repérer et réparer les problèmes de suivi

\subsubsection n Les images sont noires
C'est que les images ne sont pas acquises correctement en amont : redémarrez le module VIAM (ou CAMERA).

\subsubsection o Le suivi ne fonctionne pas du tout
\li Vérifiez les images : l'image affichée en haut doit être celle provenant de la caméra gauche du robot (mettez une main devant la caméra en question pour voir). Si ce n'est pas le cas, modifiez les paramètres d'initialisation de VIAM (ou échangez les câbles des caméras, mais c'est à éviter).
\li Ce n'est toujours pas réglé ? Alors ça doit être un problème de triangulation. Vérifiez que les paramètres de calibrations lus par VIAM sont corrects (refaites les au besoin) et vérifiez les matrices de passages : la matrice globale est définie en dur dans detectionParams.h, la baseline (distance entre les caméras) est redéfinie dans le main.cpp ou le gestRunTaskCodels.cc (suivant les informations de VIAM).

\subsubsection p Le suivi fonctionne mal
\li Vérifiez les couleurs! Le module étant très dépendant de ces dernières, il faut vérifier qu'elles ne soient pas déréglées. Si elles le sont réellement, vous pourrez le voir à l'oeil nu : les images doivent refléter ce que vous voyez avec vos yeux! Coriander (ou les paramètres d'initialisation de VIAM) vous permettra d'y remédier. Si les images ne vous choquent pas, vérifiez la carte de vraisemblance de couleur peau qui peut être affichée facilement : elle devrait être blanche aux endroits de couleur chaire et noire ailleurs. Si les réglage de couleur de l'image vous semblent bon, mais que le module ne voit pas correctement la couleur peau, il va falloir refaire un apprentissage de cette dernière (c'est à dire acquérir des images de cette couleur et les traiter hors ligne, voir main.cpp).
\li Vérifiez l'environnement. Si vous avez derrière vous une grande armoire de couleur chaire, ou si votre pull est de cette couleur, ça ne va pas aider le traqueur...

\subsubsection q Les mains ont tendance à disparaitre ou ont du mal à réapparaitre
\li Votre pull ne les couvre-t-il pas trop ? Remontez un peu les manches pour voir...
\li Il est possible que le traqueur échange parfois les mains gauches et droites. Or, si votre main gauche est vue comme une main droite, le traqueur aura du mal à réinitialiser sa main gauche, puisqu'il la cherchera à votre gauche. Mais vous pouvez l'aider en mettant brièvement les mains derrière le dos pour lui permettre ensuite de réinitialiser les deux mains correctement.

*/
