/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

#ifndef GESTTRIANGULATION_H
#define GESTTRIANGULATION_H

/**
 * \author BURGER Brice
 * \since 11/2006
 */
 
#include "mathutils.h"
#include "cv.h"
#include "detectionParams.h"
using namespace std;



/** Cette fonction initialise l'espace de travail de la fonction de triangulation :
 * 
 */
void initTriangulation();

/** Cette fonction relache l'espace de travail de la fonction de triangulation :
 * 
 */
void finTriangulation();


/** Cette fonction apparie et triangule des blobs :
 * \param nbBlobsG : nombre de blobs 2D côté gauche
 * \param nbBlobsD : nombre de blobs 2D côté droit
 * \param moyG : moyennes des blobs 2D du côté gauche
 * \param moyD : moyennes des blobs 2D du côté droit
 * \param covG : covariances des blobs 2D du côté gauche
 * \param covD : covariances des blobs 2D du côté droit
 * \param bavard : si FALSE, la fonction ne dit rien, sinon elle est bavarde
 * 
 * \return on renvoie le nombre de blobs triangulés, ainsi que moy3D et cov3D : moyennes et covariances des blobs triangulés
 * 
 * pour ne trianguler que des points (sans covariances), il suffit d'entrer NULL pour covG, covD et cov3D
 */
int triangulationBlobs (bool bavard, int nbBlobsG, int** moyG, double** covG, int nbBlobsD, int** moyD, double** covD, int** moy3D, double** cov3D);



/************************************************/
/*  fonctions utilisées pour la triangulation : */
/************************************************/  

/** Cette fonction apparie deux listes de points :
 * \param dist1 : liste de distances entres voisins du côté 1
 * \param dist2 : liste de distances entres voisins du côté 2
 * \param nb1 : nombre de points du côté 1
 * \param nb2 : nombre de points du côté 2
 * \param possibiliteappariement : possibilités d'appariements
 * 
 * \return on renvoie la liste des appariements du côté 1 avec le 2
 */
void creerAppariement(int *appariement, int ***dist1, int nb1, int ***dist2, int nb2, int **possibiliteappariement);

/** Cette fonction calcule la distance des voisins de p1[I] par rapport au voisins de p2[J] via leurs listes de distances entre voisins :
 * \param I : numéro du point étudié côté 1
 * \param J : numéro du point étudié côté 2
 * \param dist1i : liste de distances de p1[i] à ses voisins
 * \param dist2j : liste de distances de p2[j] à ses voisins
 * \param nb1 : nombre de points du côté 1
 * \param nb2 : nombre de points du côté 2
 * 
 * \return on renvoie la distance calculée
 */
int getNeighBorDistance(int I, int **dist1i, int nb1, int J, int **dist2j, int nb2);

/** Cette fonction recoupe deux liste d'appariements :
 * \param appariement1 : liste d'appariement d'un côté
 * \param appariement2 : liste d'appariement de l'autre côté
 * \param nb1 : taille de appariement1
 * \param nb2 : taille de appariement2
 * 
 * \return on renvoie la liste des appariements complets appariements
 */
void recouperAppariements(int *appariements, int *appariement1, int nb1, int *appariement2, int nb2);

#endif
