/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

#ifndef DETECTION_PARAMS_H
#define DETECTION_PARAMS_H

/** detectionParams.h
 * 
 * Ce fichier contient les paramètres par défaut et structures communes des détections.
 * 
 * Toute nouvelle variable globale doit être déclarée en extern dans la première partie du fichier, puis déclarée normalement et initialisée dans la seconde.
 * 
 * \author BURGER Brice
 * \since 03/2007
 */

#include <string>
using std::string;
#include <cv.h>
#include <highgui.h>
#include <iostream>
#include "../../gestConst.h"


/** nombre de blobs peau maximum */
#define BLOBS_PEAU_MAX 50
/** nombre de visages maximum */
#define VISAGE_MAX 10
/** distance maximum et minimum au delà de laquelle on ne cherche pas de blob (et on refuse les particules) (en mm) */
#define DISTMAXRECHERCHE 3500
#define DISTMINRECHERCHE 500



/** structure stoquant une liste de rectangles */
typedef struct listeRect
{
	int nbRect;//nombre de rectangles
	CvRect rect[VISAGE_MAX];//tableau de rectangles
}listeRect;



#ifndef GLOBALE //si on n'est pas dans le main ou dans le RunTaskCodels, les variables sont déclarées externes

/* paramètres de la détection */
extern int surfaceminblob;//surface minimum d'un blob peau 2D
extern int seuilbas;//seuil sous lequel la proba est ramenée à 0 dans l'image de proba peau
extern int nbfermeture;//nb de d'itération de fermeture appliquée pour améliorer l'image de proba peau (i.e. nb dilations, suivi de nb érosions)

/* calibration */
extern double Mdg[];//matrice de passage de la camera de droite vers celle de gauche	
extern double Mgd[];//matrice de passage de la camera de gauche vers celle de droite	
extern double intrinsicsLeft[];//parametres intrinseques : u0, v0, au, av
extern double intrinsicsRight[];//centre et taille des pixels

/* chemin de la base du code */
extern string BASE;//chemin de base (pour rechercher des fichiers)

/* taille des images à traiter */
extern int w;
extern int h;

/* paramètres de la triangulation */
extern int distSecuTriangulation;//valeur en pixels du décalage permis entre les blobs de deux images par rapport à la logique

/* paramètres des mesures */
extern int limiteImgDist;//limite autour de laquelle on binarise l'image de contours

/* volume d'une main */
extern int demivolumeMain;//on considère que le volume d'une main est constant

/* régulation de la taille des blobs */
extern int tailleBlobMin;//taille min d'un demi-axe
extern int tailleBlobMax;//taille max d'un demi-axe

/* espace de couleur */
extern int espaceCoul;/* baseRGB 0 ; baseBGR 1; baseYCrCb 2; baseHSV 3; baseI1I2I3 4; baseIrg 5; basergbN 6; baseLab 7 */


#else/*GLOBALE*/ //si on est dans le main ou dans le RunTaskCodels, les variables sont déclarées et initialisées



/* paramètres de la détection */
int surfaceminblob = 20;//surface minimum d'un blob peau 2D (en nombre de pixels)
int seuilbas = 170;//seuil sous lequel la proba est ramenée à 0 dans l'image de proba peau
int nbfermeture = 0;//nb de d'itération de fermeture appliquée pour améliorer l'image de proba peau (i.e. nb dilations, suivi de nb érosions)


/* calibration */// ATTENTION : les trois premières lignes de la dernière colonne sont habituellement en mètres! (on veut des mm)
double Mdg[] = {	0.9999,		0.0117,		0.0021,		270.3175,
			-0.0117,	0.9999,		0.0084,		-004.5170,
			-0.0020,	-0.0084,	1.0000,		-005.1121,
			0,		0,		0,		1.0000};//matrice de passage de la camera de droite vers celle de gauche

double Mgd[] = {	1.0000,		-0.0117,	-0.0020,	-270.4,
			0.0117,		0.9999,		-0.0084,	001.3,
			0.0021,		0.0084,		0.9999,		004.6,
			0,		0,		0,		1.0000};//matrice de passage de la camera de gauche vers celle de droite

double intrinsicsLeft[] = {155, 125 ,310 ,310};//parametres intrinseques : u0, v0, au, av
double intrinsicsRight[] = {175, 125 ,310 ,310};//centre et taille des pixels

/* chemin de la base du code */
string BASE = "/usr/local/openrobots/modules/gest/code/";//chemin de base (pour rechercher des fichiers)

/* taille de l'image en entrée du traqueur */
int w = 320;
int h = 240;

/* paramètres de la triangulation */
int distSecuTriangulation = 10;//valeur en pixels du décalage permis entre les blobs de deux images par rapport à la logique

/* paramètres des mesures */
int limiteImgDist = 50;//limite autour de laquelle on binarise l'image de contours

/* volume d'une main */
int demivolumeMain = 62500;//on considère que le volume constant d'une main est d'environ 20cm*10cm*2.5cm = 500000 mm³, soit pour des demi-axes : 10cm*5cm*1.25cm = 62500 mm³

/* régulation de la taille des blobs */
int tailleBlobMin = 10;//taille min d'un demi-axe (en mm) (ATTENTION, doit être !=0 !!!)
int tailleBlobMax = 150;//taille max d'un demi-axe (en mm)

/* espace de couleur */
int espaceCoul = 0;/* baseRGB 0 ; baseBGR 1; baseYCrCb 2; baseHSV 3; baseI1I2I3 4; baseIrg 5; basergbN 6; baseLab 7 */



/** fonctions pour mettre en place de nouveaux paramètres de calibration
 * 
 * \param u0,v0 : centre de l'image
 * \param au,av : taille des pixels
 * \param hauteurImage,largeurImage : taille de l'image d'origine de ces paramètres (important s'il y a eu redimensionnement)
 */
inline void setintrinsicsLeft(double u0, double v0, double au, double av, int hauteurImage, int largeurImage) {
	intrinsicsLeft[0]=u0*w/(double)largeurImage;
	intrinsicsLeft[1]=v0*h/(double)hauteurImage;
	intrinsicsLeft[2]=au*w/(double)largeurImage;
	intrinsicsLeft[3]=av*h/(double)hauteurImage;
	printf("nouveaux paramètres intrinsèques (image gauche) :\n");
	printf("u0=%3.3f , v0=%3.3f , au=%3.3f , av=%3.3f\n",intrinsicsLeft[0],intrinsicsLeft[1],intrinsicsLeft[2],intrinsicsLeft[3]);
}
inline void setintrinsicsRight(double u0, double v0, double au, double av, int hauteurImage, int largeurImage) {
	intrinsicsRight[0]=u0*w/(double)largeurImage;
	intrinsicsRight[1]=v0*h/(double)hauteurImage;
	intrinsicsRight[2]=au*w/(double)largeurImage;
	intrinsicsRight[3]=av*h/(double)hauteurImage;
	printf("nouveaux paramètres intrinsèques (image droite) :\n");
	printf("u0=%3.3f , v0=%3.3f , au=%3.3f , av=%3.3f\n",intrinsicsRight[0],intrinsicsRight[1],intrinsicsRight[2],intrinsicsRight[3]);
}



#endif/*GLOBALE*/

#endif
