/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

/* mesureDistance.cpp
 * 
 * Ce fichier contient les fonctions nécessaires au calcul d'une image de distance (i.e. distance par rapport aux contours) et des opérations sur cette image.
 * 
 * Auteur : BURGER Brice
 * Date : 04/2007
 */
#include "mesureDistance.h"




/* Constructeur */
MesureDistance::MesureDistance() {

	CvSize imgSize=cvSize(w,h);
	
	/* image calculée */
	dist = cvCreateImage(imgSize,IPL_DEPTH_8U,1);
	
	/* image de travail */
	imgNG = cvCreateImage(imgSize,IPL_DEPTH_8U,1);//traduction de l'image courante RGB en NG
	imgtemp = cvCreateImage(imgSize,IPL_DEPTH_32F,1);//image temporaire pour le calcul de l'image de distance qui doit être en 32F
}


/* Destructeur */
MesureDistance::~MesureDistance() {
	
	cvReleaseImage(&dist);
	cvReleaseImage(&imgNG);
	cvReleaseImage(&imgtemp);
}


/* Remise À Zéro d'une image
 * 
 * cette fonction est nécessaire quand on travail sur des images avec ROI :
 * on ne calcule alors que sur certaines parties de l'image, le reste de l'image ne correspondant donc à rien
 * (c'est donc essentiellement pour l'affichage qu'il faut faire une RAZ)
 */
void MesureDistance::RAZ() {
	
	cvResetImageROI(dist);
	cvZero(dist);
}


/* calcule l'image de mouvement
 *
 * img : image courante (RGB ou NG)
 */
void MesureDistance::calculeImgDist(IplImage* img) {
	
	/* gestion des ROI */
	rect = cvGetImageROI(img);
	cvSetImageROI(dist, rect);
	cvSetImageROI(imgNG, rect);
	cvSetImageROI(imgtemp, rect);
	
	if (img->nChannels==3) {
		/* conversion de l'image courante en niveau de gris */
		cvCvtColor(img, imgNG, CV_RGB2GRAY);
		cvSobel(imgNG, dist, 1, 1, 3);
	}
	else cvSobel(img, dist, 1, 1, 3);
	
	/* on binarise l'image */
	cvThreshold(dist, dist, limiteImgDist, 255, CV_THRESH_BINARY_INV);//la limite est éventuellement à revoir
	
	/* on calcule l'image de distance *///la distance est calculée par rapport aux pixels à 0
	cvDistTransform(dist, imgtemp, CV_DIST_L2, 3);
	
	/* on convertie l'image 32F en 8U */
	cvConvertScale(imgtemp, dist, 1, 0);
}


