/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

/* mesureDistance3D.cpp
 * 
 * Ce fichier contient les fonctions nécessaires au calcul d'une image de distance à partir de coordonnées 3D.
 * 
 * Auteur : BURGER Brice
 * Date : 04/2007
 */
#include "mesureDistance3D.h"




/* Constructeur */
MesureDistance3D::MesureDistance3D() {

	/* création des mesures 2D */
	mesureDistanceG = new MesureDistance();
	mesureDistanceD = new MesureDistance();
}


/* Destructeur */
MesureDistance3D::~MesureDistance3D() {
	
	delete mesureDistanceG;
	delete mesureDistanceD;
}


/* obtention de l'image de distance Gauche
 * 
 */
IplImage* MesureDistance3D::getImgDistG() {
	
	cvResetImageROI(mesureDistanceG->dist);
	return mesureDistanceG->dist;
}


/* obtention de l'image de distance Droite
 * 
 */
IplImage* MesureDistance3D::getImgDistD() {
	
	cvResetImageROI(mesureDistanceD->dist);
	return mesureDistanceD->dist;
}



/* calcule les images de distance gauche et droite dans la zone indiquée
 *
 * imgG,imgD : images à traiter
 * zones : zones 3D de recherche du visage (sous forme des rectangles projetés sur les images) (NULL pour calculer sur toute l'image)
 */
void MesureDistance3D::calculeImgDist(IplImage* imgG, IplImage* imgD, CvRect* zones) {
	
	if (zones) {
		/* on désigne une ROI dans chaque image pour ne rechercher que dans la zone voulue */
		cvSetImageROI(imgG, zones[0]);
		cvSetImageROI(imgD, zones[1]);
	}
	
	/* calcul */
	mesureDistanceG->calculeImgDist(imgG);
	mesureDistanceD->calculeImgDist(imgD);
	
	if (zones) {
		/* on nettoie les images des ROI pour que çà reste transparent au reste du programme */
		cvResetImageROI(imgG);
		cvResetImageROI(imgD);
	}
}


