/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

/* mesureMvt.cpp
 * 
 * Ce fichier contient les fonctions nécessaires au calcul d'une image de mouvement et des opérations sur cette image.
 * 
 * Auteur : BURGER Brice
 * Date : 03/2007
 */
#include "mesureMvt.h"




/* Constructeur */
MesureMvt::MesureMvt() {

	CvSize imgSize=cvSize(w,h);
	
	/* image calculée */
	mvt = cvCreateImage(imgSize,IPL_DEPTH_8U,1);
	
	/* images de travail */
	imgprec = cvCreateImage(imgSize,IPL_DEPTH_8U,1);//image précédente (traduite en niveau de gris)
	imgcour = cvCreateImage(imgSize,IPL_DEPTH_8U,1);//image courante (traduite en niveau de gris)
	cvZero(imgcour);//pour avoir une image de mouvement même le premier tour (et éviter certains problèmes)
}


/* Destructeur */
MesureMvt::~MesureMvt() {
	
	cvReleaseImage(&mvt);
	cvReleaseImage(&imgprec);
	cvReleaseImage(&imgcour);
}


/* Remise À Zéro d'une image
 * 
 * cette fonction est nécessaire quand on travail sur des images avec ROI :
 * on ne calcule alors que sur certaines parties de l'image, le reste de l'image ne correspondant donc à rien
 * (c'est donc essentiellement pour l'affichage qu'il faut faire une RAZ)
 */
void MesureMvt::RAZ() {
	
	cvResetImageROI(mvt);
	cvZero(mvt);
}


/* calcule l'image de mouvement
 *
 * img : image courante
 * nouveau : si true, il s'agit d'une nouvelle image, sinon on calcule sur la même image mais dans une ROI différente
 */
void MesureMvt::calculeImgMvt(IplImage* img, bool nouveau) {
	
	if (nouveau) {
		/* l'image courante devient la précédente */
		imgtemp = imgprec;
		imgprec = imgcour;
		imgcour = imgtemp;
	}
	
	/* gestion des ROI */
	rect = cvGetImageROI(img);
	rectgrand.x = rect.x - rect.width/2;// zone de calcul agrandie
	rectgrand.y = rect.y - rect.height/2;
	rectgrand.width = rect.width*2;
	rectgrand.height = rect.height*2;
	cvSetImageROI(imgprec, rect);
	cvSetImageROI(mvt, rect);
	
	
	/* conversion de l'image courante en niveau de gris */
	cvSetImageROI(imgcour, rectgrand);//on convertit au-delà de la zone de calcul en prévision de la prochaine image (la zone de calcul se sera déplacée)
	cvSetImageROI(img, rectgrand);
	cvCvtColor(img, imgcour, CV_RGB2GRAY);
	cvSetImageROI(imgcour, rect);
	cvSetImageROI(img, rect);

	/* calcul de l'image de mouvement = soustraction des images */
	cvSub(imgcour,imgprec, mvt);//mvt = imgcour - imgprec
}


