/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

/* mesureMvt3D.cpp
 * 
 * Ce fichier contient les fonction nécessaires au calcul des images de mouvements à partir de coordonnées 3D.
 * 
 * Auteur : BURGER Brice
 * Date : 03/2007
 */
#include "mesureMvt3D.h"


/* Constructeur */
MesureMvt3D::MesureMvt3D() {

	/* création des mesures 2D */
	mesureMvtG = new MesureMvt();
	mesureMvtD = new MesureMvt();
}


/* Destructeur */
MesureMvt3D::~MesureMvt3D() {
	
	delete mesureMvtG;
	delete mesureMvtD;
}


/* obtention de l'image de mouvement Gauche
 * 
 */
IplImage* MesureMvt3D::getImgMvtG() {
	
	cvResetImageROI(mesureMvtG->mvt);
	return mesureMvtG->mvt;
}


/* obtention de l'image de mouvement Droite
 * 
 */
IplImage* MesureMvt3D::getImgMvtD() {
	
	cvResetImageROI(mesureMvtD->mvt);
	return mesureMvtD->mvt;
}



/* calcule les images de mouvement gauche et droite dans la zone indiquée
 *
 * nouveau : si true, il s'agit d'une nouvelle image, sinon on calcule sur la même image mais dans une ROI différente
 * imgG,imgD : images à traiter
 * zones : zones 3D de recherche du visage (sous forme des rectangles projetés sur les images) (NULL pour calculer sur toute l'image)
 */
void MesureMvt3D::calculeImgMvt(bool nouveau, IplImage* imgG, IplImage* imgD, CvRect* zones) {

	if (zones) {
		/* on désigne une ROI dans chaque image pour ne rechercher que dans la zone voulue */
		cvSetImageROI(imgG, zones[0]);
		cvSetImageROI(imgD, zones[1]);
	}
	
	/* calcul */
	mesureMvtG->calculeImgMvt(imgG, nouveau);
	mesureMvtD->calculeImgMvt(imgD, nouveau);
	
	if (zones) {
		/* on nettoie les images des ROI pour que çà reste transparent au reste du programme */
		cvResetImageROI(imgG);
		cvResetImageROI(imgD);
	}
}



