/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

#ifndef DETECTIONVISAGE_H
#define DETECTIONVISAGE_H

/**
 * \author BURGER Brice
 * \since 02/2007
 */
 
#include <cv.h>
#include <highgui.h>

#include "detectionParams.h"
#include "triangulation.h"
#include "projection.h"
#include "faceDetector.h"


class detectionVisage {


	public:

	/** Constructeur */
	detectionVisage();

	/** Destructeur */
	~detectionVisage();

	/** détection de visages 3D
	 * 
	 * \param imgG et imgD sont les images a traiter (gauche et droite)
	 * 
	 * \return visages est la liste des visages renvoyees (position du centre de la sphère)
	 * \return on renvoie également le nombre de visages contenus dans la liste de visages renvoyée
	 */
	int detecte(IplImage* imgG, IplImage* imgD, int** visages);
	
	/** détection de visages 3D : 
	 * 
	 * on recherche ici dans une zone
	 * 
	 * \param imgG et imgD sont les images a traiter (gauche et droite)
	 * \param zones : zones 3D de recherche du visage (sous forme des rectangles projetés sur les images) (NULL pour calculer sur toute l'image)
	 * 
	 * \return visages est la liste des visages renvoyees (position du centre de la sphère)
	 * \return on renvoie également le nombre de visages contenus dans la liste de visages renvoyée
	 */
	int detecteAvecZones(IplImage* imgG, IplImage* imgD, int** visages, CvRect* zones);

	/** détection de visages 3D : 
	 * 
	 * on recherche ici dans une zone qui est sensée ne contenir qu'un seul visage (si plusieurs sont détectés, seul le premier sera renvoyé)
	 * 
	 * \param imgG et imgD sont les images a traiter (gauche et droite)
	 * \param zones : zones 3D de recherche du visage (sous forme des rectangles projetés sur les images) (NULL pour calculer sur toute l'image)
	 * 
	 * \return visage est le visage renvoyé (position du centre de la sphère)
	 * \return On renvoie également si un visage a été détecté ou non.
	 */
	bool detecteUnSeul(IplImage* imgG, IplImage* imgD, int* visage, CvRect* zones);



	private:
	
	/* détecteurs de base */
	faceDetector *detecteurVisageG, *detecteurVisageD;
	
	/* rectangle qui nous intéresse */
	listeRect visagesG;
	listeRect visagesD;
	CvRect* rect;
	
	/* pour la triangulation de points */
	int **p1, **p2;
	int** p3D;
	
	/* pour la projection */
	int rayon;
	CvPoint point2dgauche, point2ddroite;
	
	/* divers */
	int nbpt3D;//nb de points 3D obtenus après triangulation
	int i,j;//itérations

};
#endif
