/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

/* faceDetector.cpp
 * Auteur : BURGER Brice
 * Date : 02/2007
 */
#include "faceDetector.h"


/* Constructeur
 * 
 * nomfichierclassifieurxml : le nom du fichier xml dans lequel est stocké l'apprentissage de visages
 */
faceDetector::faceDetector(string nomfichierclassifieurxml) {
	
	/* allocation de la mémoire */
	mem = cvCreateMemStorage(0);
  
	/* creation de la cascade de classifieur pour la detection de visages à partir d'un fichier xml */
	string chemin = BASE + "fichiers_donnees/detectors/haarcascades/" + nomfichierclassifieurxml;
	cout << "faceDetector.cpp : chargement de " << chemin << endl;

	cascade = (CvHaarClassifierCascade*)cvLoad(chemin.c_str(), 0, 0, 0 );
}


/* Destructeur */
faceDetector::~faceDetector() {

	cvReleaseMemStorage(&mem);
	cvReleaseHaarClassifierCascade(&cascade);
}


/* Éxécution de la détection de visage : 
 * 
 * cette fonction trouve le ou les visages dans l'image donnée en entrée
 * cette fonction trouve le ou les visages dans l'image donnée en entrée
 * elle renvoie un une structure contenant les rectangles extraits
 */
void faceDetector::trouve(IplImage* img, listeRect* visages) {

	/* RAZ de la mémoire */
	cvClearMemStorage(mem);
	
	/* Détection des visages */
	//visagesSeq = cvHaarDetectObjects(img, cascade, mem, 1.2, 2, CV_HAAR_DO_CANNY_PRUNING,cvSize(0,0));
	visagesSeq = cvHaarDetectObjects(img, cascade, mem, 1.1, 2, CV_HAAR_DO_CANNY_PRUNING,cvSize(0,0));//plus lent, mais plus efficace
	
	/* mise en forme + mise à l'échelle si nécessaire*/
	if (visagesSeq->total<VISAGE_MAX) visages->nbRect = visagesSeq->total;
	else {
		visages->nbRect = VISAGE_MAX;
		printf("ATTENTION: faceDetector.cpp : trop de visages (%d)\n",visagesSeq->total);
	}
	for(int i=0;i<visages->nbRect;i++) {
		rect = (CvRect*)cvGetSeqElem(visagesSeq,i);
		visages->rect[i].x = rect->x;
		visages->rect[i].y = rect->y;
		visages->rect[i].width = rect->width;
		visages->rect[i].height = rect->height;
	}
}


