/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

/* traqueur.cpp
 * 
 * Programme principal, il fait l'interface entre le main ou genom et le reste du code.
 * Ici sont implémentés les fonctions de haut niveau du traqueur 3D à filtre particulaire.
 * Le but de ce programme est de traquer une main dans l'espace (forme ellipsoidale) à partir des images d'un banc stéréo.
 * 
 * Il a été modifié pour suivre en priorité une tête, dont il déduit la main. Les modification aportées sont annotées avec //§§§ .
 * 
 * ATTENTION : ce fichier n'est plus à jour!
 * 
 * Auteur : BURGER Brice
 * Date : 01/2007
 */


#include "traqueur.h"


/* constructeur du traqueur 3D :
 * 
 * imgG et imgD : images représentatives de celles qui serviront ensuite durant la traque (même taille, même convention de couleur, ...)
 */
Traqueur::Traqueur(IplImage* imgG, IplImage* imgD) {

	/* taille de l'image */
	w = imgG->width;
	h = imgG->height;
	
	particules = new Particules(nbvarEllipsoide);//MAINdROITE//§§§
	particulesTETE = new Particules(nbvarSphere);//TETE//§§§
	
	/* création des détecteurs */
	detecteurBlob = new detectionBlobPeau(imgG,imgD);
	detecteurVisage = new detectionVisage(imgG,imgD);

	projection = new double[3];
	g = new double[nbpart];
	gTETE = new double[nbpart];
	
	/* selene */
	p1 = new flottant*[BLOBS_PEAU_MAX];
	p2 = new flottant*[BLOBS_PEAU_MAX];
	for (int i=0; i<BLOBS_PEAU_MAX; i++) {
		p1[i] = new flottant[2];
		p2[i] = new flottant[2];
	}
	
	imgTmpg2 = cvCreateImage(cvSize(w,h),8,3);//§§§
	imgTmpd2 = cvCreateImage(cvSize(w,h),8,3);//§§§
	
	imgTmpg = cvCreateImage(cvSize(w,h),8,3);
	imgTmpd = cvCreateImage(cvSize(w,h),8,3);
	img = cvCreateImage(cvSize(w,h),8,3);
	
	nomsauvegarde = new char[100];//pour sauvegarder des images
	
	/* images NG pour pouvoir afficher l'image de proba peau */
	for (int i=0;i<3;i++) imageNG[i] = cvCreateImage(cvSize(w,h),8,1);
	
	AVEC_INTERFACE = false;
	AFFICHE_PARTICULES = false;
	SAUVEGARDE = false;
	
	numero=0;
}


/* destructeur du traqueur 3D :
 *
 */
Traqueur::~Traqueur() {
	
	delete particules;
	delete particulesTETE;//§§§
	delete detecteurBlob;
	delete detecteurVisage;
	
	delete[] projection;
	delete[] g;
	delete[] gTETE;
	
	delete &rectG;
	delete &rectD;
	
	for (int i=0; i<BLOBS_PEAU_MAX; i++) {
		delete[] p1[i];
		delete[] p2[i];
	}
	delete[] p1;
	delete[] p2;
	
	if (AVEC_INTERFACE) {
		delete afficheur;
		cvReleaseImage(&img);
	}
	
	for (int i=0;i<3;i++) cvReleaseImage(&imageNG[i]);
	cvReleaseImage(&imgTmpg2);//§§§
	cvReleaseImage(&imgTmpd2);//§§§
}


/*
 * fonctions pour changer la matrice de covariance de la dynamique
 * 
 * typeParticule : type de particule qu'on veut affecter par cette modification
 * cov : nouvelle matrice de covariance (ATTENTION : la taille de la matrice donnée doit correspondre à celle modifiée)
 * 
 * retourne true s'il n'y a pas eu de problème, false autrement
 */
bool Traqueur::changecov(TYPE_PARTICULE_GEST typeParticule, double *cov) {//§§§
	
	switch (typeParticule) {
		
/*		case MAINgAUCHE:
			
			
*/			
		case MAINdROITE:
			return ( particules->changecov(cov) );
		case TETE:
			return ( particulesTETE->changecov(cov) );
		default:
			cout << "traqueur.cpp (changecov) : ERREUR, type de particule (" << typeParticule << ") inexistant" << endl;
			return false;
	}
}


/* si l'on veut utiliser une interface graphique :
 * 
 */
void Traqueur::avec_interface(bool oui) {

	if (oui) {
	
		if (AVEC_INTERFACE) return;
	
		if (AFFICHE_PARTICULES) {
			img = cvCreateImage(cvSize(w,h*3),8,3);
			afficheur = new ImgDisplay(w, h*3, ":0", 0, 0 );
		}
		else {
			img = cvCreateImage(cvSize(w,h*2),8,3);
			afficheur = new ImgDisplay(w, h*2, ":0", 0, 0 );
		}
	
		AVEC_INTERFACE = true;
		
	} else {
		
		if (!AVEC_INTERFACE) return;
		
		delete afficheur;
		cvReleaseImage(&img);
		
		AVEC_INTERFACE = false;
	}
}


/* si l'on veut afficher les particules :
 * 
 */
void Traqueur::affiche_particules(bool oui) {
	
	if (oui) {
		
		if (AFFICHE_PARTICULES) return;
		
		AFFICHE_PARTICULES = true;
		
		if (AVEC_INTERFACE) {//alors il faut relancer l'affichage
			avec_interface(false);
			avec_interface(true);
		}
		
	} else {
		
		if (!AFFICHE_PARTICULES) return;
		
		AFFICHE_PARTICULES = false;
		
		if (AVEC_INTERFACE) {//alors il faut relancer l'affichage
			avec_interface(false);
			avec_interface(true);
		}
	}
}


/* si l'on veut sauvegarder les images obtenues :
 * 
 */
void Traqueur::sauvegarde(bool oui) {
	
	if (oui) SAUVEGARDE = true;
	else SAUVEGARDE = false;
}


/* fonction d'initialisation globale du traqueur :
 * 
 * imgG,imgD : images à traiter (gauche et droite)
 * type : type d'initialisation :
 *		**0 : renvoie le premier blob couleur peau détecté
 * 		**1 : avec zone de recherche précisée (si l'on connait une position approximative de la main à traquer)
 * 		**2 : ici on initialise sur la main droite à partir d'une détection de tête
 * 			La main droite est un blob peau situé à moins de 1.5 m de la tête et se situant à gauche de celle-ci (et en bas de la tête).
 * 			ATTENTION : il ne doit y avoir qu'un seul visage !! Sinon c'est le premier qui est pris.
 * x,y,z,rayon : cube dans lequelle on cherche un blob peau pour le type 1
 * 
 * retourne la particule initiale trouvée ou NULL si on a rien trouvé
 */
double* Traqueur::INIT(IplImage* imgG, IplImage* imgD, int type, double x, double y, double z, double rayon) {
	
	switch (type) {

		case 0:
			cout << "INITIALISATION sur le premier blob vu" << endl;
			break;
		case 1:
			cout << "INITIALISATION d'après indications" << endl;
			break;
		case 2:
			cout << "INITIALISATION sur la main droite" << endl;
			break;
		default:
			cout << "ERREUR : mauvais type d'initialisation" << endl;
			return NULL;
	}
	
	int i;
	
	detecteurVisage->detecte(imgG, imgD, &rectG);
	
	if (type==2) if ((rectG.nbRect==0)||(rectD.nbRect==0)) {
		cout << "Aucun visage detecte" << endl;
		if (AVEC_INTERFACE) {
			cvLine( imgG, cvPoint(0,0), cvPoint(w,h), CV_RGB(0,0,0),4);
			cvLine( imgD, cvPoint(0,0), cvPoint(w,h), CV_RGB(0,0,0),4);
			affiche(imgG, imgD);
		}
		return NULL;
	}
	
	particules->nbpartdetectees = detecteurBlob->detecteAvecLimitations(false, imgG, imgD, particules->particuleDetectees, DISTMAXRECHERCHE, 0, &rectG, &rectD);
	
	if (AVEC_INTERFACE) {
		
		/************** on reprojette l'ellipsoide 3D sur les images pour visualiser la particule initiale *************/
		for (i=0;i<particules->nbpartdetectees;i++) {
			projettePoint (particules->particuleDetectees[i], &point2dgauche, &point2ddroite);//projection du centre
			projectionEllipsoide (particules->particuleDetectees[i], projection);
			projetteAxes (projection, &axesg, &axesd);
			angleg = projection[0];
			angled = angleg;//c'est approximatif, mais ça suffit largement et fait gagner du temps
		
			cvEllipse(imgG,point2dgauche,axesg,angleg,0,360,CV_RGB(255,0,0),2);
			cvEllipse(imgD,point2ddroite,axesd,angled,0,360,CV_RGB(255,0,0),2);
		}
		
		/* affichage des visages + position de leur centre */
		if (type==2) {
			for (i=0;i<rectG.nbRect;i++) {
				/* à gauche */
				rect = &(rectG.rect[i]);
				cvRectangle(imgG,cvPoint(rect->x,rect->y),cvPoint(rect->x+rect->width, rect->y+rect->height),CV_RGB(255,0,0),2);
				p1[i][0] = rect->x + rect->width/2;//on en profite pour calculer la position du centre du visage
				p1[i][1] = rect->y + rect->height/2;
			}

			for (i=0;i<rectD.nbRect;i++) {
				/* à droite */
				rect = &(rectD.rect[i]);
				cvRectangle(imgD,cvPoint(rect->x,rect->y),cvPoint(rect->x+rect->width, rect->y+rect->height),CV_RGB(255,0,0),2);
				p2[i][0] = rect->x + rect->width/2;//on en profite pour calculer la position du centre du visage
				p2[i][1] = rect->y + rect->height/2;
			}
		}
		
		/* préparation de l'affichage de l'image de proba peau */
		if (AFFICHE_PARTICULES) {
			cvCopyImage(detecteurBlob->getImgProbaPeauG(false), imageNG[0]);
			cvCopyImage(imageNG[0], imageNG[1]);
			cvCopyImage(imageNG[0], imageNG[2]);
			cvMerge(imageNG[0],imageNG[1],imageNG[2],NULL,imgTmpg);
		}
		
	} else {//Pas d'interface
	
		/* calcul de la position du centre des visages */
		for (i=0;i<rectG.nbRect;i++) {
			/* à gauche */
			rect = &(rectG.rect[i]);
			p1[i][0] = rect->x + rect->width/2;//on en profite pour calculer la position du centre du visage
			p1[i][1] = rect->y + rect->height/2;
		}

		for (i=0;i<rectD.nbRect;i++) {
			/* à droite */
			rect = &(rectD.rect[i]);
			p2[i][0] = rect->x + rect->width/2;//on en profite pour calculer la position du centre du visage
			p2[i][1] = rect->y + rect->height/2;
		}
	}

	if (particules->nbpartdetectees==0) {
		cout << "aucune particule detectee" << endl;
		if (AVEC_INTERFACE) {
			cvLine( imgG, cvPoint(0,0), cvPoint(w,h), CV_RGB(0,0,0),4);
			cvLine( imgD, cvPoint(0,0), cvPoint(w,h), CV_RGB(0,0,0),4);
			if (AFFICHE_PARTICULES) affiche(imgG, imgD, imgTmpg);
			else affiche(imgG, imgD);
		}
		return NULL;//si on ne detecte rien
	}
	
	particules->invnbpartdetectees = 1/(double)particules->nbpartdetectees;
	
	/* choix de la particule détectée qui servira d'initialisation : */
	int j;
	double min = 100;
	i=-1;
	double	xmin,xmax,ymin,ymax,zmin,zmax;
	switch (type) {

		case 0:
			for (j=0;j<particules->nbpartdetectees;j++) {
				//if (min > particules->particuleDetectees[j][0]) { min=particules->particuleDetectees[j][0]; i=j; }//choix de l'ellipsoide (le plus a gauche(a droite de l'image) = min sur x)
				//if (min > particules->particuleDetectees[j][1]) { min=particules->particuleDetectees[j][1]; i=j; }//choix de l'ellipsoide (le plus en haut = min sur y)
				if (min > particules->particuleDetectees[j][2]) { min=particules->particuleDetectees[j][2]; i=j; }//choix de l'ellipsoide (le plus en avant = min sur z)
			}
			break;

		case 1:
			xmin=x-rayon;
			xmax=x+rayon;
			ymin=y-rayon;
			ymax=y+rayon;
			zmin=z-rayon;
			zmax=z+rayon;

			for (int j=0;j<particules->nbpartdetectees;j++) {
				if ((particules->particuleDetectees[j][0] < xmax)&&(particules->particuleDetectees[j][0] > xmin)) {
					if ((particules->particuleDetectees[j][1] < ymax)&&(particules->particuleDetectees[j][1] > ymin)) {
						if ((particules->particuleDetectees[j][2] < zmax)&&(particules->particuleDetectees[j][2] > zmin)) i=j;//choix de l'ellipsoide, si plusieurs correspondent, on prend le dernier qui vient
					}
				}
			}

			if (i==-1) {
				cout << "Aucun blob 3D correspondant a l'indication, initialisation annulee" << endl;
				if (AVEC_INTERFACE) {
					cvLine( imgG, cvPoint(0,0), cvPoint(w,h), CV_RGB(0,0,0),4);
					cvLine( imgD, cvPoint(0,0), cvPoint(w,h), CV_RGB(0,0,0),4);
					if (AFFICHE_PARTICULES) affiche(imgG, imgD, imgTmpg);
					else affiche(imgG, imgD);
				}
				return NULL;
			}
			
			break;

		case 2:
			iris::set2DBlobs(p1,NULL,1,p2,NULL,1);//fait la triangulation de la tête -> on en déduit la main droite
			
			if (iris::getnb3D()==0) {
				cout << "pas de visage conserve apres triangulation" << endl;
				if (AVEC_INTERFACE) {
					cvLine( imgG, cvPoint(0,0), cvPoint(w,h), CV_RGB(0,0,0),4);
					cvLine( imgD, cvPoint(0,0), cvPoint(w,h), CV_RGB(0,0,0),4);
					if (AFFICHE_PARTICULES) affiche(imgG, imgD, imgTmpg);
					else affiche(imgG, imgD);
				}
				return NULL;
			}
			//on ne prend qu'un seul visage (on utilise que le premier de la liste pour l'instant)//...
			xmin=iris::getp3d(0)[0]-1.5;
			xmax=iris::getp3d(0)[0];
			ymin=iris::getp3d(0)[1];
			ymax=iris::getp3d(0)[1]+1.5;
			zmin=iris::getp3d(0)[2]-1.5;
			zmax=iris::getp3d(0)[2]+1.5;
			
			for (int j=0;j<particules->nbpartdetectees;j++) {
				if ((particules->particuleDetectees[j][0] < xmax)&&(particules->particuleDetectees[j][0] > xmin)) {
					if ((particules->particuleDetectees[j][1] < ymax)&&(particules->particuleDetectees[j][1] > ymin)) {
						if ((particules->particuleDetectees[j][2] < zmax)&&(particules->particuleDetectees[j][2] > zmin)) i=j;//choix de l'ellipsoide, si plusieurs correspondent, on prend le dernier qui vient
					}
				}
			}
			
			if (i==-1) {
				cout << "pas de main droite trouvee" << endl;
				if (AVEC_INTERFACE) {
					cvLine( imgG, cvPoint(0,0), cvPoint(w,h), CV_RGB(0,0,0),4);
					cvLine( imgD, cvPoint(0,0), cvPoint(w,h), CV_RGB(0,0,0),4);
					if (AFFICHE_PARTICULES) affiche(imgG, imgD, imgTmpg);
					else affiche(imgG, imgD);
				}
				return NULL;
			}
			
			break;
	}
	
	/* main à suivre */
	memcpy(particules->particuleFinale, particules->particuleDetectees[i], nbvarEllipsoide*sizeof(double));
	/* tête à suivre *///§§§
	particulesTETE->particuleFinale[0] = iris::getp3d(0)[0];
	particulesTETE->particuleFinale[1] = iris::getp3d(0)[1];
	particulesTETE->particuleFinale[2] = iris::getp3d(0)[2];
	

	if (AVEC_INTERFACE) {
		/************** on reprojette l'ellipsoide 3D sur les images pour visualiser la particule initiale *************/
		projettePoint (particules->particuleFinale, &point2dgauche, &point2ddroite);//projection du centre
		projectionEllipsoide (particules->particuleFinale, projection);
		projetteAxes (projection, &axesg, &axesd);
		angleg = projection[0];
		angled = angleg;//c'est approximatif, mais ça suffit largement et fait gagner du temps
		
		cvEllipse(imgG,point2dgauche,axesg,angleg,0,360,CV_RGB(255,0,255),2);
		cvEllipse(imgD,point2ddroite,axesd,angled,0,360,CV_RGB(255,0,255),2);
		
		/************** on reprojette la tête sur les images pour visualiser la particule initiale *************///§§§
		projettePoint (particulesTETE->particuleFinale, &point2dgauche, &point2ddroite);//projection du centre
		
		cvCircle( imgG, point2dgauche, int (RayonTete*intrinsicsLeft[2]/particulesTETE->particuleFinale[2]), CV_RGB(0,255,255), 2 );//pour simplifier, on néglige les déformations de l'image et on affiche un cercle
		cvCircle( imgD, point2ddroite, int (RayonTete*intrinsicsLeft[2]/particulesTETE->particuleFinale[2]), CV_RGB(0,255,255), 2 );

		if (AFFICHE_PARTICULES) affiche(imgG, imgD, imgTmpg);
		else affiche(imgG, imgD);
	}
		
	/* la particule initiale est l'équivalent d'une particule parfaite (poids 1, aucune autre particule conservée) */
	memcpy(particules->particulesPrec[0],particules->particuleFinale,nbvarEllipsoide*sizeof(double));
	particules->nbpartconservees = 1;
	particules->poidsPrec[0] = 1;
	/* idem pour la tête *///§§§
	memcpy(particulesTETE->particulesPrec[0],particulesTETE->particuleFinale,nbvarSphere*sizeof(double));
	particulesTETE->nbpartconservees = 1;
	particulesTETE->poidsPrec[0] = 1;
	
	cout << "init faite : ON PEUT COMMENCER" << endl;
	
	return particules->particuleFinale;
}


/* fonction de réinitialisation du traqueur : 
 * 
 * avantSelection : précise si la sélection a déjà eu lieu ou non (pour savoir si la particuleFinale à déjà été écrasée ou non)
 * 
 * Ici on essaie de sauver un pas défectueux pour éviter une réinitialisation globale (via INIT) faisant perdre la continuité de la traque.
 * 
 * retourne la particule initiale trouvée ou NULL si on a rien trouvé
 */
double* Traqueur::REINIT(IplImage* imgG, IplImage* imgD, bool avantSelection) {
	
	double	xmin,xmax,ymin,ymax,zmin,zmax;
	if (avantSelection) {
		cout << "tentative de REINITIALISATION douce (cause : toutes les evaluations etaient a 0)" <<endl;
		
		xmin=particules->particuleFinale[0]-distmaxReinit;
		xmax=particules->particuleFinale[0]+distmaxReinit;
		ymin=particules->particuleFinale[1]-distmaxReinit;
		ymax=particules->particuleFinale[1]+distmaxReinit;
		zmin=particules->particuleFinale[2]-distmaxReinit;
		zmax=particules->particuleFinale[2]+distmaxReinit;
		
	} else {
		cout << "tentative de REINITIALISATION douce  (cause : la particule finale etait très mauvaise)" <<endl;
		
		xmin=particules->particuleFinalePrec[0]-distmaxReinit;
		xmax=particules->particuleFinalePrec[0]+distmaxReinit;
		ymin=particules->particuleFinalePrec[1]-distmaxReinit;
		ymax=particules->particuleFinalePrec[1]+distmaxReinit;
		zmin=particules->particuleFinalePrec[2]-distmaxReinit;
		zmax=particules->particuleFinalePrec[2]+distmaxReinit;
	}
	
	/* on cherche une particule pas trop éloignée de la particule du tour précédent */
	int i=-1;
	for (int j=0;j<particules->nbpartdetectees;j++) {
		if ((particules->particuleDetectees[j][0] < xmax)&&(particules->particuleDetectees[j][0] > xmin)) {
			if ((particules->particuleDetectees[j][1] < ymax)&&(particules->particuleDetectees[j][1] > ymin)) {
				if ((particules->particuleDetectees[j][2] < zmax)&&(particules->particuleDetectees[j][2] > zmin)) i=j;//choix de l'ellipsoide, si plusieurs correspondent, on prend le dernier qui vient
			}
		}
	}

	if (i==-1) {
		cout << "Aucun blob 3D correspondant dans la zone de recherche, reinitialisation annulee" << endl;
		if (AVEC_INTERFACE) {
			cvLine( imgG, cvPoint(0,0), cvPoint(w,h), CV_RGB(0,0,0),4);
			cvLine( imgD, cvPoint(0,0), cvPoint(w,h), CV_RGB(0,0,0),4);
			if (AFFICHE_PARTICULES) affiche(imgG, imgD, imgTmpg);
			else affiche(imgG, imgD);
		}
		return NULL;
	}
	
	memcpy(particules->particuleFinale, particules->particuleDetectees[i], nbvarEllipsoide*sizeof(double));

	if (AVEC_INTERFACE) {
		/************** on reprojette l'ellipsoide 3D sur les images pour visualiser la particule initiale *************/
		projettePoint (particules->particuleFinale, &point2dgauche, &point2ddroite);//projection du centre
		projectionEllipsoide (particules->particuleFinale, projection);
		projetteAxes (projection, &axesg, &axesd);
		angleg = projection[0];
		angled = angleg;//c'est approximatif, mais ça suffit largement et fait gagner du temps
		
		cvEllipse(imgG,point2dgauche,axesg,angleg,0,360,CV_RGB(255,0,255),2);
		cvEllipse(imgD,point2ddroite,axesd,angled,0,360,CV_RGB(255,0,255),2);

		if (AFFICHE_PARTICULES) affiche(imgG, imgD, imgTmpg);
		else affiche(imgG, imgD);
	}
				
	cout << "reinitialisation faite : ON PEUT RECOMMENCER" << endl;
	
	/* la particule pseudo-initiale est l'équivalent d'une particule parfaite (poids 1, aucune autre particule conservée) */
	memcpy(particules->particulesPrec[0],particules->particuleFinale,nbvarEllipsoide*sizeof(double));
	particules->nbpartconservees = 1;
	particules->poidsPrec[0] = 1;
	
	return particules->particuleFinale;
}


/* fonction de réinitialisation du traqueur (tête) : 
 * 
 * avantSelection : précise si la sélection a déjà eu lieu ou non (pour savoir si la particuleFinale à déjà été écrasée ou non)
 * 
 * Ici on essaie de sauver un pas défectueux pour éviter une réinitialisation globale (via INIT) faisant perdre la continuité de la traque.
 * 
 * retourne true si la réinitialisation s'est bien passée, false autrement
 */
bool Traqueur::REINITtETE(IplImage* imgG, IplImage* imgD, bool avantSelection) {//§§§
	
	double	xmin,xmax,ymin,ymax,zmin,zmax;
	
	if (avantSelection) {
		cout << "tentative de REINITIALISATION douce de la TETE (cause : toutes les evaluations etaient a 0)" <<endl;
		
		xmin=particulesTETE->particuleFinale[0]-distmaxReinit;
		xmax=particulesTETE->particuleFinale[0]+distmaxReinit;
		ymin=particulesTETE->particuleFinale[1]-distmaxReinit;
		ymax=particulesTETE->particuleFinale[1]+distmaxReinit;
		zmin=particulesTETE->particuleFinale[2]-distmaxReinit;
		zmax=particulesTETE->particuleFinale[2]+distmaxReinit;
		
	} else {
		cout << "tentative de REINITIALISATION douce de la TETE (cause : la particule finale etait très mauvaise)" <<endl;
		
		xmin=particulesTETE->particuleFinalePrec[0]-distmaxReinit;
		xmax=particulesTETE->particuleFinalePrec[0]+distmaxReinit;
		ymin=particulesTETE->particuleFinalePrec[1]-distmaxReinit;
		ymax=particulesTETE->particuleFinalePrec[1]+distmaxReinit;
		zmin=particulesTETE->particuleFinalePrec[2]-distmaxReinit;
		zmax=particulesTETE->particuleFinalePrec[2]+distmaxReinit;
	}
	
	if (particulesTETE->nbpartdetectees==0) {
		cout << "Aucun visage n'a ete detecte, reinitialisation annulee" << endl;
		return false;
	}
	
	/* on cherche une particule pas trop éloignée de la particule du tour précédent */
	int i=-1;
	for (int j=0;j<particulesTETE->nbpartdetectees;j++) {
		if ((particulesTETE->particuleDetectees[j][0] < xmax)&&(particulesTETE->particuleDetectees[j][0] > xmin)) {
			if ((particulesTETE->particuleDetectees[j][1] < ymax)&&(particulesTETE->particuleDetectees[j][1] > ymin)) {
				if ((particulesTETE->particuleDetectees[j][2] < zmax)&&(particulesTETE->particuleDetectees[j][2] > zmin)) i=j;//choix de l'ellipsoide, si plusieurs correspondent, on prend le dernier qui vient
			}
		}
	}

	if (i==-1) {
		cout << "Aucun visage correspondant dans la zone de recherche, reinitialisation annulee" << endl;
		return false;
	}
	
	memcpy(particulesTETE->particuleFinale, particulesTETE->particuleDetectees[i], nbvarSphere*sizeof(double));
				
	cout << "reinitialisation douce faite" << endl;
	
	/* la particule pseudo-initiale est l'équivalent d'une particule parfaite (poids 1, aucune autre particule conservée) */
	memcpy(particulesTETE->particulesPrec[0],particulesTETE->particuleFinale,nbvarSphere*sizeof(double));
	particulesTETE->nbpartconservees = 1;
	particulesTETE->poidsPrec[0] = 1;
	
	return true;
}


/* fonction évaluant une particule via les fonctions d'évaluation 2D :
 * 
 * particule : la particule à évaluer
 * 
 * renvoie son évaluation (entre 0 et 1)
 */
double Traqueur::EVAL(double *particule) {//§§§faudra changer en EVALellipsoide
	
	projettePoint (particule, &point2dgauche, &point2ddroite);//projection du centre
	projectionEllipsoide (particule, projection);
	projetteAxes (projection, &axesg, &axesd);
	angleg = projection[0];
	angled = angleg;//c'est approximatif, mais ça suffit largement et fait gagner du temps
	
	double eval;
	eval = proximitepeauEllipse(detecteurBlob->getImgProbaPeauG(false), point2dgauche, axesg, angleg);
	eval *= proximitepeauEllipse(detecteurBlob->getImgProbaPeauD(false), point2ddroite, axesd, angled);
	
	return eval;
}


/* fonction évaluant une particule via les fonctions d'évaluation 2D :
 * 
 * particule : la particule à évaluer
 * 
 * renvoie son évaluation (entre 0 et 1)
 */
double Traqueur::EVALtete(double *particule) {//§§§
	
	projettePoint (particule, &point2dgauche, &point2ddroite);//projection du centre
	int rayon = int (RayonTete*intrinsicsLeft[2]/particule[2]);
	
	double eval;
	eval = proximitepeauCercle(detecteurBlob->getImgProbaPeauG(false), point2dgauche, rayon);
	eval *= proximitepeauCercle(detecteurBlob->getImgProbaPeauD(false), point2ddroite, rayon);
	
	return eval;
}


/* pour avancer d'un pas dans le filtre particulaire :
 * 
 */
double* Traqueur::PAS(IplImage* imgG, IplImage* imgD) {
	
	int i;
	gmax = 0;
	gmaxTETE = 0;//§§§
	int Xg,Yg,Xd,Yd;//mémoire du rectangle/visage : coin supérieur haut du rectangle de l'image dans laquelle on cherche un visage
	
	
	/* on recherche les visages dans les environs de la détection de visage précédente */
	//rectG et rectD contiennent encore le visage précédemment retenu //...ATTENTION, faudra changer çà quand on traquera réellement la tête
	cvReleaseImage(&imgTmpg2);
	cvReleaseImage(&imgTmpd2);
	rect = &(rectG.rect[0]);
	Xg=rect->x+rect->width*(1-2)/2;
	Yg=rect->y+rect->height*(1-2)/2;
	imgTmpg2 = cvCreateImage(cvSize(rect->width*2,rect->height*2),8,3);//voir si *2 çà suffit (faire une variable dans params !)//...
	cvGetRectSubPix( imgG, imgTmpg2, cvPoint2D32f( rect->x+rect->width/2 , rect->y+rect->height/2 ) );
	rect = &(rectD.rect[0]);
	Xd=rect->x+rect->width*(1-2)/2;
	Yd=rect->y+rect->height*(1-2)/2;
	imgTmpd2 = cvCreateImage(cvSize(rect->width*2,rect->height*2),8,3);
	cvGetRectSubPix( imgD, imgTmpd2, cvPoint2D32f( rect->x+rect->width/2 , rect->y+rect->height/2 ) );
	
	detecteurVisageG->trouve(imgTmpg2, &rectG);
	detecteurVisageD->trouve(imgTmpd2, &rectD);
	
	
	/* on interdit les visages */
	for(i=0;i<rectG.nbRect;i++) {//pour tous les visages repérés à gauche
		rect = &(rectG.rect[i]);
		//il faut encore remettre les rectangles à leur place
		rect->x+=Xg;
		rect->y+=Yg;
		p1[i][0] = rect->x + rect->width/2;
		p1[i][1] = rect->y + rect->height/2;
	}
	for(i=0;i<rectD.nbRect;i++) {//pour tous les visages repérés à droite
		rect = &(rectD.rect[i]);
		//il faut encore remettre les rectangles à leur place
		rect->x+=Xd;
		rect->y+=Yd;
		p2[i][0] = rect->x + rect->width/2;
		p2[i][1] = rect->y + rect->height/2;
	}
	iris::set2DBlobs(p1,NULL,rectG.nbRect,p2,NULL,rectD.nbRect);//fait la triangulation
	double doubletemp[] = {iris::getp3d(i)[0],iris::getp3d(i)[1],iris::getp3d(i)[2]};
	for (i=0;i<iris::getnb3D();i++) particules->interdire (doubletemp , RayonTete+5);//+5 pour en pas prendre la peau du coup
	//... il faudra encore modifier les interdiction : en faisant en sorte que chache type de particule interdise les autres, ce sera générique//§§§
	
	cout << "nb de visages detectes : " << iris::getnb3D() << endl;//§§§
	
	/* la détection de visage vient d'avoir lieu *///§§§
	particulesTETE->nbpartdetectees = iris::getnb3D();
	for (i=0;i<iris::getnb3D();i++) {//...faudra faire une fonction dans detection.cpp pour çà
		particulesTETE->particuleDetectees[i][0] = iris::getp3d(i)[0];
		particulesTETE->particuleDetectees[i][1] = iris::getp3d(i)[1];
		particulesTETE->particuleDetectees[i][2] = iris::getp3d(i)[2];
	}
	if (particulesTETE->nbpartdetectees!=0) particulesTETE->invnbpartdetectees = 1/(double)particulesTETE->nbpartdetectees;//§§§
	
	/* détection des blobs peau */
	particules->nbpartdetectees = detecteurBlob->detecteAvecLimitations(false, imgG, imgD, particules->particuleDetectees, DISTMAXRECHERCHE, 0, &rectG, &rectD);
	
	if (particules->nbpartdetectees!=0) particules->invnbpartdetectees = 1/(double)particules->nbpartdetectees;
	
	if (AFFICHE_PARTICULES) if (AVEC_INTERFACE) {
		cvCopyImage(detecteurBlob->getImgProbaPeauG(false), imageNG[0]);
		cvCopyImage(imageNG[0], imageNG[1]);
		cvCopyImage(imageNG[0], imageNG[2]);
		cvMerge(imageNG[0],imageNG[1],imageNG[2],NULL,imgTmpg);
	}
	
	/* pour chaque particule */
	for(i=0;i<nbpart;i++) {
		
		/* Prédiction */
		lamda = particules->genere(i);
		lamdaTETE = particulesTETE->genere(i);//§§§
		
		/* on élimine les particules interdites */
		//if (particules->interdit(i)) lamda=0;//...c'est tout le temps interdit !
		if (particulesTETE->interdit(i)) lamdaTETE=0;//§§§
		
		/* calcul de la vraisemblance */
		if (lamda!=0) g[i] = EVAL(particules->particules[i]);
		if (lamdaTETE!=0) gTETE[i] = EVALtete(particulesTETE->particules[i]);//§§§

		g[i] *= lamda;
		gTETE[i] *= lamdaTETE;//§§§
		
		if (gmax<g[i]) gmax=g[i];
		if (gmaxTETE<gTETE[i]) gmaxTETE=gTETE[i];//§§§
	}
	
	if (gmax==0) /*return*/ REINIT(imgG, imgD, true);//si ce pas a été très mauvais, on essaye de le récupérer
	if (gmaxTETE==0) /*return*/ REINITtETE(imgG, imgD, true);//si ce pas a été très mauvais, on essaye de le récupérer//...à prendre en compte encore//çà va pas marcher, faut prendre en compte quand il n'y a pas de détection //§§§

	/* calcul du poids */
	for(i=0;i<nbpart;i++) {
		particules->poids[i] = poids(g[i],gmax);
		particulesTETE->poids[i] = poids(gTETE[i],gmaxTETE);//§§§
		
		/* affichage des particules ayant un poids non nul */
		if (particules->poids[i]!=0) if (AFFICHE_PARTICULES) if (AVEC_INTERFACE) {
			projettePointgauche (particules->particules[i], &point2dgauche);
			projectionEllipsoide (particules->particules[i], projection);
			projetteAxesgauche (projection, &axesg);
			angleg = projection[0];

			cvEllipse(imgTmpg,point2dgauche,axesg,angleg,0,360,CV_RGB((int)(255*particules->poids[i]),255,0),2);
		}
		
		if (particulesTETE->poids[i]!=0) if (AFFICHE_PARTICULES) if (AVEC_INTERFACE) {//§§§
			projettePointgauche (particulesTETE->particules[i], &point2dgauche);//projection du centre

			cvCircle(imgTmpg, point2dgauche, int (RayonTete*intrinsicsLeft[2]/particulesTETE->particules[i][2]), CV_RGB((int)(255*particulesTETE->poids[i]),255,255), 2 );//pour simplifier, on néglige les déformations de l'image et on affiche un cercle
		}
	}
	
	/* sélection de la particule finale */
	particules->selectionne(nbpart);
	particulesTETE->selectionne(nbpart);//§§§

	/* évaluation du résultat */
	if (EVAL(particules->particuleFinale) < 0.01) /*return*/ REINIT(imgG, imgD, false);//si ce pas a été très mauvais, on essaye de le récupérer
	if (EVALtete(particulesTETE->particuleFinale) < 0.01) /*return*/ REINITtETE(imgG, imgD, false);//si ce pas a été très mauvais, on essaye de le récupérer//...faut encore le prendre en compte//çà va pas marcher, faut prendre en compte quand il n'y a pas de détection//§§§

	/* rééchantillonnage */
	particules->reechantillonne();
	particulesTETE->reechantillonne();//§§§

	if (AVEC_INTERFACE) {
		/************** on reprojette l'ellipsoide 3D sur les images pour visualiser la particule initiale *************/
		projettePoint (particules->particuleFinale, &point2dgauche, &point2ddroite);//projection du centre
		projectionEllipsoide (particules->particuleFinale, projection);
		projetteAxes (projection, &axesg, &axesd);
		angleg = projection[0];
		angled = angleg;//c'est approximatif, mais ça suffit largement et fait gagner du temps
		
		cvEllipse(imgG,point2dgauche,axesg,angleg,0,360,CV_RGB(0,0,255),2);
		cvEllipse(imgD,point2ddroite,axesd,angled,0,360,CV_RGB(0,0,255),2);
		
		/************** on reprojette la tête sur les images pour visualiser la particule initiale *************///§§§
		projettePoint (particulesTETE->particuleFinale, &point2dgauche, &point2ddroite);//projection du centre
		
		cvCircle( imgG, point2dgauche, int (RayonTete*intrinsicsLeft[2]/particulesTETE->particuleFinale[2]), CV_RGB(255,255,255), 2 );//pour simplifier, on néglige les déformations de l'image et on affiche un cercle
		cvCircle( imgD, point2ddroite, int (RayonTete*intrinsicsLeft[2]/particulesTETE->particuleFinale[2]), CV_RGB(255,255,255), 2 );


		if (AFFICHE_PARTICULES) affiche(imgG, imgD, imgTmpg);
		else affiche(imgG, imgD);
	}

	return particules->particuleFinale;
}





/**********************/
/* fonctions internes */
/**********************/


/* affichage sur l'ecran du robot : 2 images l'une en dessous de l'autre */
void Traqueur::affiche (IplImage* imgg, IplImage* imgd) {

	if (!AVEC_INTERFACE) return;

	memcpy(img->imageData, imgg->imageData, w*h*3*sizeof(char));
	memcpy(img->imageData + (w*h*3*sizeof(char)), imgd->imageData, w*h*3*sizeof(char));

	afficheur->PutImage(img);
		
	if (SAUVEGARDE) {
		numero++;
		sprintf(nomsauvegarde,"/tmp/gestImages/image%d.png",numero);
		cvSaveImage(nomsauvegarde, img);
	}
}


/* affichage sur l'ecran du robot : 3 images l'une en dessous de l'autre */
void Traqueur::affiche (IplImage* imgg, IplImage* imgd, IplImage* img3) {

	if (!AVEC_INTERFACE) return;
	
	memcpy(img->imageData, imgg->imageData, w*h*3*sizeof(char));
	memcpy(img->imageData + (w*h*3*sizeof(char)), imgd->imageData, w*h*3*sizeof(char));
	memcpy(img->imageData + (2*w*h*3*sizeof(char)), img3->imageData, w*h*3*sizeof(char));

	afficheur->PutImage(img);

	if (SAUVEGARDE) {
		numero++;
		sprintf(nomsauvegarde,"/tmp/gestImages/image%d.png",numero);
		cvSaveImage(nomsauvegarde, img);
	}
}

