/* Copyright (c) Mathias Fontmarty (LAAS-CNRS) */
#ifndef MATHUTILS_H
#define MATHUTILS_H

#include <iostream>

typedef double flottant;


class C3DPoint
{
public:
  C3DPoint();
  C3DPoint(flottant x, flottant y, flottant z, 
	   int id = -1, flottant taille = 0);
  C3DPoint(flottant x, flottant y, flottant z, 
	   flottant nx, flottant ny, flottant nz, 
	   int id = -1, flottant taille = 0);
  
  flottant x,y,z;
  flottant nx,ny,nz;
  flottant taille;
  int id;

};

struct S2DPoint
{
  flottant x,y,prof;
  flottant nx,ny;
  flottant taille;
  int      visible;
  int      id;
};

struct S2DSegment
{
  S2DPoint first;
  S2DPoint second;
};





/* Various Utils
 ***************************/
#define absd(X) ( (X)>0?(X):-(X))

/*
 * Matrix Operations          
 ********************************/


void matrix4Mult(flottant *out, flottant *in1, flottant *in2);
void matrixMult(flottant *vOut, flottant *vIn1, flottant *vIn2, int vIn1dim1, int vIn1dim2, int vIn2dim2);

void matrix4Display(flottant * mat, char* nom="Matrice");
void matrixDisplay(flottant *mat, int nbLignes, int nbColonnes, char *nom="Matrice");

void matrix4Transpose(flottant *vOut, flottant *vIn);
void matrixTranspose(flottant *vOut, flottant *vIn, int dimIn1, int dimIn2);

void matrix4Add(flottant *vOut, flottant *vIn1, flottant *vIn2);
void matrixAdd(flottant *vOut, flottant *vIn1, flottant *vIn2, int dim1, int dim2);

void matrixInvert(flottant *inv, flottant *mat, int dim);

flottant matrix3Determinant(flottant *mat);



void matrix4VectorMult(flottant *vOut, flottant *mIn, flottant *vIn);
void matrix3VectorMult(flottant *vOut, flottant *mIn, flottant *vIn, int dim=3);
void vector3Mult(flottant *vOut, flottant *vIn1, flottant *vIn2);
void vectorAdd(flottant *vOut, flottant *vIn1, flottant *vIn2, int dim);
void vectorSub(flottant *vOut, flottant *vIn1, flottant *vIn2, int dim);





int segmentIntersect(S2DPoint *A, S2DPoint *B, S2DPoint *C, S2DPoint *D,  S2DPoint *pointIntersection);

void trierPointsParX(S2DPoint **tableauATrier, S2DPoint **tableauTrie, int tailleTableau);

void clamp(int *toBeClamped, int infLimit, int supLimit);


void triangulate(int *pointTriangule, flottant *erreur,
		 int ug, int vg, flottant u0g, 
		 flottant v0g, flottant aug, flottant avg,
		 int ud, int vd, flottant u0d, 
		 flottant v0d, flottant aud, flottant avd,
		 flottant *Mdg,
		 flottant *covp1 = NULL, flottant *covp2 = NULL,
		 flottant *covP = NULL);


#endif
