/* Copyright (c) Ludovic Brethes (LAAS-CNRS) */
#include "video4linux.h"


video4linux::video4linux(int w,int h)
{
  this->width=w;
  this->height=h;
  image=NULL;
  
  camfd=open("/dev/video0",O_RDWR);
  if (ioctl(camfd,VIDIOCGCAP,&vcap)==-1){
    perror("GCAP");
    exit(1);
  }
  
  int val = -1;

  ioctl(camfd,VIDIOCPWCFACTORY,&val); // restore factory defaults
  struct pwc_leds LED;
  /* Blinking LED, 1 second interval, 20% duty cycle */
  LED.led_on = 200;
  LED.led_off = 800;
  ioctl(camfd, VIDIOCPWCSLED, &LED);

  for (int i=0;i<vcap.channels;i++){
    Channel.channel=i;
    if((ioctl(camfd, VIDIOCGCHAN, &Channel) != -1))
      {
	
	Channel.norm = 1;
	Channel.channel = i;
	
	if((ioctl(camfd,VIDIOCSCHAN, &Channel) != -1)&&
	   (ioctl(camfd, VIDIOCGCAP, &vcap)    != -1))
	  {
	    if(Channel.type&VIDEO_TYPE_CAMERA)
	      {
		printf("--> video4linux (constructor) \t:\t Camera %s, channel %s\n",vcap.name,Channel.name);
		
	      }
	    
	  }			
      }
  }
  
  printf("--> video4linux (constructor) \t:\t Contructor done\n");
  k=0;
}

video4linux::~video4linux(){
  
  cvReleaseImage(&image);
  munmap(memorymap,mbuf.size);
  close(camfd);
  printf("video closed\n");
}

/** No descriptions */
bool video4linux::init()
{
  CvSize size;	
  if (ioctl(camfd,VIDIOCGPICT,&picture)==-1)
    perror("GPICT");
  picture.palette=VIDEO_PALETTE_YUV420P;
  picture.depth=24;
  //   picture.brightness=30000;
  //   picture.hue=30000;
  //   picture.colour=30000;
  //   picture.contrast=30000;
  //   picture.whiteness=30000;
  if (ioctl(camfd,VIDIOCSPICT,&picture)==-1)
    perror("SPICT");
  
  vmmap.frame=0;
  vmmap.width=width;
  vmmap.height=height;
  vmmap.format=picture.palette;
  
  size.width=this->width;
  size.height=this->height;
  
  if(ioctl(camfd, VIDIOCGMBUF,
	   &mbuf) == -1){
    perror("mbuf");
    exit(1);
  }
  memorymap = (char*)mmap(0,
			  mbuf.size,
			  PROT_READ|PROT_WRITE, MAP_SHARED,
			  camfd,
			  0);
  
  
  if(ioctl(camfd, VIDIOCMCAPTURE,
	   &vmmap) == -1)
    {
      if(ioctl(camfd, VIDIOCSYNC,
	       vmmap.frame )==-1)
	{
	  fprintf(stderr, "icvVideoInit: ioctl VIDIOCMCAPTURE and "
		  "VIDIOCSYNC failed \n");
	  //return 0;
	}//if ioctl(..., VIDIOCSYNK, ...)
      printf("CMCAP FAILLED\n");
    }
  
  this->image = cvCreateImageHeader(size,IPL_DEPTH_8U,3);
  this->width = size.width;
  this->height = size.height;
  this->rgbimg = cvCreateImage(cvSize(this->image->width,this->image->height),IPL_DEPTH_8U,3);
  
  //Init des tables de conversion yuv2rgb
  InitConvtTbl();
  
 return true;
}

/** No descriptions */
bool video4linux::grab()
{
  vmmap.frame=1;
  if(ioctl(camfd,VIDIOCMCAPTURE,&vmmap)==-1){
    perror("MMAP F1");
    exit(1);
  }
  
  // sync
  if (ioctl(camfd,VIDIOCSYNC,&vmmap.frame)==-1){
    perror("SYNC");
    exit(1);
  }
  SetRGBfromYUV420(this->image,(void *)(memorymap+mbuf.offsets[1]));
  return true;  
}

int video4linux::SetRGBfromYUV420(IplImage * image, void*memorymap)
{
  unsigned char * rgbdata,*srcdata;
  
  if (image ==NULL){
    printf("image ==NULL\n");
    return -1;
  }
  if (memorymap == NULL){
    printf("mmap==NULL");
    return -1;
  }

 //Conversion 
 rgbdata=(unsigned char*)this->rgbimg->imageData;
 srcdata=(unsigned char *) memorymap;
 YUV2BGR420(srcdata,rgbdata,width,height);

 //Recopie dans l'image
  cvSetImageData(this->image,this->rgbimg->imageData,this->rgbimg->widthStep);
  return 0; /* ok */
}
