/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

#ifndef _gestSTRUCT_H
#define _gestSTRUCT_H

#include "gestConst.h"

/* ATTENTION : toutes les distances sont maintenant en mm (et non plus en mètres) */

/////////////////////////////////////////////////////////////
//////////////////     ATTENTION!!!       ///////////////////
// les structures doivent être alignée sur des doubles!!   //
// (1 double = 2 int = 8 char = 8 octets)                  //
// d'où les unused                                         //
/////////////////////////////////////////////////////////////




/* énumérations */

typedef enum ETAT_GEST_GENOM	//état du système
{
	RIEN_GENOM=0,			//rien n'est traqué
	TOUT_GENOM=1,			//tout est traqué
	PASmAINgAUCHE_GENOM=2,		//tout est traqué sauf la main gauche
	PASmAINdROITE_GENOM=3,		//tout est traqué sauf la main droite
	AUCUNEmAIN_GENOM=4,		//seule la tête est traquée
	PAStETE_GENOM=5,		//tout est traqué sauf la tête
	PAStETEnImAINgAUCHE_GENOM=6,	//seule la main droite est traquée
	PAStETEnImAINdROITE_GENOM=7	//seule la main gauche est traquée
}ETAT_GEST_GENOM;

typedef enum TYPE_PARTICULE_GEST_GENOM//type de particule
{
	MAINgAUCHE_GENOM=0,
	MAINdROITE_GENOM=1,
	TETE_GENOM=2
}TYPE_PARTICULE_GEST_GENOM;

typedef enum GEST_BOOLEEN		//pour permettre de se servir des booléen dans gest.gen
{
	GEST_False=0,
	GEST_True=1
}GEST_BOOLEEN;

typedef enum GEST_DISPLAY_PARAMS
{
	withoutInterface=0,	//sans interface graphique
	withSimpleInterface=1,	//avec interface graphique simple (sans image supplémentaire)
	SkinProba=2,		//interface + image de probabilité de couleur peau
	Mvt=3,			//interface + image de mouvement
	Dist=4			//interface + image de distance
}GEST_DISPLAY_PARAMS;

/* espace de couleur */
/* baseRGB 0; baseBGR 1; baseYCrCb 2; baseHSV 3; baseI1I2I3 4; baseIrg 5; basergbN 6; baseLab 7 */
typedef enum GEST_COLORSPACE
{
	baseRGB=0,
	baseBGR=1,
	baseYCrCb=2,
	baseHSV=3,
	baseI1I2I3=4,
	baseIrg=5,
	basergbN=6,
	baseLab=7
}GEST_COLORSPACE;



/* structures */

typedef struct GEST_INIT_STRUCT {//structure contenant les paramètres d'initialisation
	char CameraPosterName[GEST_TAILLE_CHAR_MAX];//nom du poster de (camera ou viam) à lire
} GEST_INIT_STRUCT;

typedef struct GEST_DIST
{
	int x;
	int y;
	int z;
	int distmin;		//distance minimum a partir de laquelle on declanche
}GEST_DIST;

typedef struct GEST_VAR
{
	int x;//paramètres de la matrice de variance
	int y;
	int z;	
	int ax;
	int ay;
	int az;
	int alpha;
	int betha;
	int gamma;
	TYPE_PARTICULE_GEST_GENOM type;//type de particules
}GEST_VAR;

/* vecteur d'état, i.e. position 3D des cibles suivies (tête et mains) */
typedef struct GEST_POS
{
	ETAT_GEST_GENOM ETAT;//état du système (indique ce qui est traqué et donc les parties du poster qui désignent la réalité)
	/* tête */
	int Tx;
	int Ty;
	int Tz;
	/* main gauche */
	int MGx;
	int MGy;
	int MGz;	
	int MGax;
	int MGay;
	int MGaz;
	int MGalpha;
	int MGbetha;
	int MGgamma;
	/* main droite */
	int MDx;
	int MDy;
	int MDz;	
	int MDax;
	int MDay;
	int MDaz;
	int MDalpha;
	int MDbetha;
	int MDgamma;
}GEST_POS;

/* projection sur l'image gauche du vecteur d'état *///pour l'instant, on ne projette que la tête. S'il y en a besoin, on rajoutera le reste à la suite.
typedef struct GEST_POS2D
{
	/* tête */
	int Tx;	//position en x de la tête dans l'image
	int Ty;	//position en y de la tête dans l'image
	int R;	//rayon de la tête dans l'image		-> R=-1 quand la tête n'est pas suivie
}GEST_POS2D;

typedef struct GEST_GRAPHIC_PARAMS
{
	GEST_DISPLAY_PARAMS display;	//choix du type d'affichage
	GEST_BOOLEEN leftBased;		//si l'affichage supplémentaire est basé sur l'image gauche (à droite sinon)	-> sert uniquement s'il y a une interface avec image supplémentaire
	GEST_BOOLEEN displayParticles;	//si on affiche les particules sur l'image supplémentaire			-> sert uniquement s'il y a une interface avec image supplémentaire
	GEST_BOOLEEN Debayerisee;		//si l'image en entrée est déjà débayérisée
}GEST_GRAPHIC_PARAMS;

typedef struct GEST_TRACKING_PARAMS
{
	GEST_COLORSPACE ColorSpace;	//choix de l'espace de couleur
	GEST_BOOLEEN ViaSequence;		//si on utilise une séquence d'images plutôt que les caméras
}GEST_TRACKING_PARAMS;

typedef struct GEST_SAVE_PARAMS
{
	char savePath[128];		//chemin vers l'endrois où l'image et/ou les résultats seront stoqués
	GEST_BOOLEEN saveImages;		//si l'on sauvegarde les images ou non
	GEST_BOOLEEN saveResults;		//si l'on sauvegarde les résultats ou non
}GEST_SAVE_PARAMS;

typedef struct GEST_WINDOW
{
	int w;
	int h;
}GEST_WINDOW;

/* contrôle de la platine */
typedef struct GEST_PLATINE_CONTROL {
	double pan;
	double tilt;
} GEST_PLATINE_CONTROL;

#endif
